#pragma once
#include <algorithm>
#include <cstdint>

#include "audio/misc/audio_buffer.h"
#include "audio/misc/circular_audio_buffer.h"
#include "misc/delegates.h"

inline static multicast_delegate<> on_latency_offset_changed;

class latency_compensator {
public:
    latency_compensator();
    ~latency_compensator();

    void init();

    void push(audio_buffer& in_buffer);
    void pop(uint64_t delta_sample, circular_buffer_vector_type& out_buffer);
    void pop(uint64_t delta_smaple, audio_buffer& out_buffer);

    /**
     * 更新缓冲区, 主要是根据全局最大延迟计算需要的buffer大小
     * @param global_max_latency 全局最大延迟
     */
    void update_buffer(int32_t min_latency, int32_t max_latency);

    void set_latency(int32_t in_latency_offset);
    void set_user_latency(int32_t in_latency_offset);

    [[nodiscard]] int32_t get_user_latency() const { return user_latency_; }
    [[nodiscard]] int32_t get_latency() const { return latency_; }
    [[nodiscard]] int32_t get_real_latency() const { return latency_ + user_latency_; }

    [[nodiscard]] circular_buffer_vector_type& get_internal_buffer() { return internal_buffer; }
protected:
    void resize(uint32_t block_size) {
        for (auto& channel : internal_buffer) {
            channel.set_capacity(block_size);
        }
    }
    // 在缓冲区前面补0, 相当于整个缓冲区向后移动
    void push_zeros(uint32_t num_samples) {
        if (num_samples == 0)
            return;
        for (auto& channel : internal_buffer) {
            channel.push_zeros(num_samples);
        }
    }
private:
    // 预计完成的sample_pos
    uint64_t finish_sample_pos_ = 0;
    int32_t user_latency_ = 0; // 用户期望延迟偏移, 大于0表示延迟, 小于0表示提前
    int32_t latency_ = 0; // 实际延迟
    circular_buffer_vector_type internal_buffer;
};
